//--------------------------------------------------------------------------
// File and Version Information:
//      created at Thu Mar  7 01:08:19 CST 2002 by newPackage.sh
// Description:
//	Class trkEx
//      Exercise to handle CDFtrack
// Environment:
//
// Author List:
//
// Copyright Information:
//
//------------------------------------------------------------------------

//------------------------
// String Header First  --
//------------------------
#include <string>

//-----------------------
// This Class's Header --
//-----------------------
#include "trkEx/trkEx.hh"
//-------------
// C Headers --
//-------------
#include <assert.h>
#include <time.h>
//---------------
// C++ Headers --
//---------------
#include <iostream.h>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "Edm/ConstHandle.hh"
#include "AbsEnv/AbsEnv.hh"

// for CDFtrack (see note 5089)
#include "TrackingObjects/Tracks/CdfTrack.hh"
#include "TrackingObjects/Storable/CdfTrackColl.hh"
#include "TrackingObjects/Storable/CdfTrackView.hh"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

static const char rcsid[] = "";

//----------------
// Constructors --
//----------------
trkEx::trkEx( const char* const theName, const char* const theDescription ) :
  AppModule( theName, theDescription )
{
}

//--------------
// Destructor --
//--------------
trkEx::~trkEx()
{
}

//--------------
// Operations --
//--------------
AppResult trkEx::beginJob( AbsEvent* aJob )
{
  char* chform_var="bulkhead:I*4,wedge:I*4,layer:I*4,side:I*4,Q:I*4,"
    "momentum:R*4,pathlength:R*4,x:R*4,y:R*4,z:R*4";
  vector<void*> var_addr;
  var_addr.push_back(&_var.bulkhead);
  var_addr.push_back(&_var.wedge);
  var_addr.push_back(&_var.layer);
  var_addr.push_back(&_var.side);
  var_addr.push_back(&_var.Q);
  var_addr.push_back(&_var.momentum);
  var_addr.push_back(&_var.pathlength);
  var_addr.push_back(&_var.x);
  var_addr.push_back(&_var.y);
  var_addr.push_back(&_var.z);

  _manager = new HepRootFileManager("trkEx.root",
                                    HepFileManager::HEP_REMOVE);
  _ntuple=&_manager->ntuple("NTUPLE",1);
  _ntuple->setColumnWise();
  _ntuple->block("var", chform_var, var_addr);

  _eventN = 0;

  cout << name( ) << " begin Job" << endl;
  return AppResult::OK;
}

AppResult trkEx::beginRun( AbsEvent* aRun )
{
  _eventN = 0;

  cout << name( ) << " begin Run" << endl;
  return AppResult::OK;
}

AppResult trkEx::event( AbsEvent* anEvent )
{
  ++_eventN;

  CdfTrackView_h trackView;
  // check if CdfTrackView storable bank is ok
  if ( CdfTrackView::defTracks(trackView)== CdfTrackView::OK ) {
    if (_eventN <=3) { cout << name() << ": CdfTrackView OK" << endl; }

    CdfTrackView::CollType& tracks = trackView->contents();
    if (_eventN <=3) {
      cout << name() << ": # of tracks= " << tracks.size() << endl;
    }

    CdfTrackView::iterator t;
    // start of track iterator
    for ( t=tracks.begin(); t!=tracks.end(); ++t) {

      if (_eventN <=3) {
	cout << name() << ": " 
	     << (*t)->id() << " " << (*t)->algorithm() << " "
	     << (*t)->numSIHits() << " " <<(*t)->momentum() << " "
	     << hex << (*t)->usedSI() << dec << endl;
      }

      // silicon hits
      if ((*t)->numSIHits()) {
	CdfTrack::SiHitIterator si;
	// do iteration over silicon hits
	for ( si=(*t)->beginSIHits(); si!=(*t)->endSIHits(); ++si) {

	  // Get path length
	  //   method getTrajectory gives trajectory of track
	  Helix traj=(*t)->getTrajectory();
	  //   Get path length along trajectory to the silicon ladder
	  double trkPL = (*si)->getArclengthAlong(traj);
	  //   Get tangent vector of the track at the silicon ladder
	  HepVector3D tan=traj.getDirection(trkPL);
	  //   Get the position of center of the silicon ladder
	  HepPoint3D sipos= (*si)->getGlobalPosition();
	  //   Make normal vector to the silicon ladder from its position
	  sipos.setZ(0.0);    // projection onto x-y plane
	  BasicVector3D normal= sipos.unit();
	  double cosTheta= normal*tan;
	  //   Theta is defined as an angle between the normal vector
	  //   of the silicon ladder and the tangent vector of the track

	  //   Get track position on silicon ladder
	  HepVector3D trkPos = traj.getPosition(trkPL);

	  // fill into ntuple
	  _var.bulkhead=(*si)->getSiDigiCode().getBarrel()*2
	    +(*si)->getSiDigiCode().getHalfLadder();
	  _var.wedge=(*si)->getSiDigiCode().getPhiWedge();
	  _var.layer=(*si)->getSiDigiCode().getLayer();
	  _var.side=(*si)->getSiDigiCode().getPnSide();
	  _var.Q=(*si)->getQtotal();
	  _var.momentum=(*t)->momentum().mag();
	  _var.pathlength=1.0/cosTheta;
	  _var.x=trkPos.x();
	  _var.y=trkPos.y();
	  _var.z=trkPos.z();

	  // capture the block 'var'
	  _ntuple->captureBlock("var");
	  _ntuple->storeCapturedData();
	  _ntuple->clearDataBlock("var");
	} // end of silicon hit iterator
      }  // end of if (silicon hit)
    } // end of track iteration
  } else {
    if (_eventN <=3) { cout << name() << ": CdfTrackView Fail" << endl; }
  }

  if (_eventN <= 3) { cout << name( ) << ": event entry" << endl; }
  return AppResult::OK;
}

AppResult trkEx::other( AbsEvent* )
{
  cout << name( ) << " other" << endl;
  return AppResult::OK;
}

AppResult trkEx::endRun( AbsEvent* aRun )
{
  cout << name( ) << " end Run" << endl;
  return AppResult::OK;
}

AppResult trkEx::endJob( AbsEvent* aJob )
{
  cout << name( ) << " end Job" << endl;
  return AppResult::OK;
}

AppResult trkEx::abortJob( AbsEvent* aJob )
{
  cout << name( ) << " abort Job" << endl;
  return AppResult::OK;
}

AppModule* trkEx::clone(const char* cloneName)
{
  return new trkEx(cloneName,"this module is a clone trkEx");
}

const char* trkEx::rcsId( ) const
{
  return rcsid;
}
